.*: Assembler messages:
.*:5: Error: operand 2 out of range `bbit032 \$23,51,foo'
.*:7: Error: operand 2 out of range `bbit0 \$23,71,foo'
.*:10: Error: operand 2 out of range `bbit132 \$23,49,foo'
.*:12: Error: operand 2 out of range `bbit1 \$23,74,foo'
.*:15: Error: operand 3 out of range `cins \$2,0,37'
.*:17: Error: operand 3 out of range `cins32 \$19,\$31,39,12'
.*:18: Error: operand 4 out of range `cins32 \$17,\$20,7,25'
.*:20: Error: operand 3 out of range `cins \$24,\$10,64,8'
.*:21: Error: operand 4 out of range `cins \$21,\$30,50,14'
.*:23: Error: opcode not supported on this processor.*
.*:24: Error: opcode not supported on this processor.*
.*:25: Error: opcode not supported on this processor.*
.*:26: Error: opcode not supported on this processor.*
.*:27: Error: opcode not supported on this processor.*
.*:28: Error: opcode not supported on this processor.*
.*:29: Error: opcode not supported on this processor.*
.*:30: Error: opcode not supported on this processor.*
.*:31: Error: opcode not supported on this processor.*
.*:32: Error: opcode not supported on this processor.*
.*:33: Error: opcode not supported on this processor.*
.*:34: Error: opcode not supported on this processor.*
.*:35: Error: opcode not supported on this processor.*
.*:36: Error: opcode not supported on this processor.*
.*:37: Error: opcode not supported on this processor.*
.*:39: Error: opcode not supported on this processor.*
.*:40: Error: opcode not supported on this processor.*
.*:41: Error: opcode not supported on this processor.*
.*:42: Error: opcode not supported on this processor.*
.*:43: Error: opcode not supported on this processor.*
.*:45: Error: operand 2 out of range `dmfc2 \$2,0x10000'
.*:46: Error: operand 2 out of range `dmtc2 \$2,0x12345'
.*:47: Error: operand 2 must be an immediate expression `dmfc2 \$9,\$12'
.*:48: Error: operand 2 must be an immediate expression `dmfc2 \$4,\$15,4'
.*:49: Error: operand 2 must be an immediate expression `dmtc2 \$16,\$8'
.*:50: Error: operand 2 must be an immediate expression `dmtc2 \$22,\$7,\$4'
.*:52: Error: operand 3 out of range `exts \$26,26,32'
.*:54: Error: operand 3 out of range `exts32 \$7,\$21,32,10'
.*:55: Error: operand 4 out of range `exts32 \$31,\$13,3,29'
.*:57: Error: operand 3 out of range `exts \$14,\$29,70,14'
.*:58: Error: operand 4 out of range `exts \$20,\$16,39,25'
.*:60: Error: operand 3 out of range `seqi \$14,\$13,512'
.*:61: Error: operand 2 out of range `seqi \$19,-771'
.*:62: Error: operand 3 out of range `snei \$18,\$30,615'
.*:63: Error: operand 2 out of range `snei \$17,-513'
