#as: --gsframe
#objdump: --sframe=.sframe
#name: SFrame supports only default return column
#...
Contents of the SFrame section .sframe:

  Header :

    Version: SFRAME_VERSION_1
    Flags: NONE
    Num FDEs: 0
    Num FREs: 0

#pass
