[^:]*: Assembler messages:
[^:]*:5: Error: 'APSR', 'CPSR' or 'SPSR' expected -- `mrs r4,apsr_nzcvq'
[^:]*:6: Error: selected processor does not support requested special purpose register -- `mrs r5,iapsr'
[^:]*:7: Error: selected processor does not support requested special purpose register -- `msr iapsr,r4'
[^:]*:8: writing to APSR without specifying a bitmask is deprecated
