#objdump: -d
#name:    
#source:  clb.s


.*:     file format elf32-s12z


Disassembly of section .text:

00000000 <.text>:
   0:	1b 91 47    	clb d0, d7
   3:	1b 91 56    	clb d1, d6
   6:	1b 91 03    	clb d2, d5
   9:	1b 91 12    	clb d3, d4
   c:	1b 91 21    	clb d4, d3
   f:	1b 91 30    	clb d5, d2
  12:	1b 91 65    	clb d6, d1
  15:	1b 91 74    	clb d7, d0
