--  Copyright 2016-2023 Free Software Foundation, Inc.
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.

with Pck; use Pck;

package body Pkg is
   procedure Flip (Bits : in out Bits_Type; I : Natural) is
   begin
      --  Create a new scope to check that the scope match algorithm is fine in
      --  the front-end.
      declare
         Rename_Subscript_Param_B : Boolean renames Bits (I);
      begin
         Rename_Subscript_Param_B := not Rename_Subscript_Param_B; -- BREAK
         Do_Nothing (Bits'Address);
      end;
   end Flip;
end Pkg;
