.*: Assembler messages:
.*:4: Warning: extended operand requested but not required
.*:5: Warning: extended operand requested but not required
.*:8: Warning: extended operand requested but not required
.*:9: Warning: extended operand requested but not required
.*:23: Warning: extended operand requested but not required
.*:24: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsll\.e \$16,\$16,8'
.*:25: Warning: extended operand requested but not required
.*:26: Warning: extended operand requested but not required
.*:28: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `ld\.e \$16,0\(\$16\)'
.*:30: Warning: extended operand requested but not required
.*:31: Warning: extended operand requested but not required
.*:32: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddiu\.e \$16,\$16,0'
.*:33: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddu\.e \$16,\$16,0'
.*:35: Warning: extended operand requested but not required
.*:36: Warning: extended operand requested but not required
.*:38: Warning: extended operand requested but not required
.*:39: Warning: extended operand requested but not required
.*:41: Warning: extended operand requested but not required
.*:42: Warning: extended operand requested but not required
.*:46: Warning: extended operand requested but not required
.*:47: Warning: extended operand requested but not required
.*:48: Warning: extended operand requested but not required
.*:49: Warning: extended operand requested but not required
.*:50: Warning: extended operand requested but not required
.*:53: Error: unrecognized extended version of MIPS16 opcode `nop\.e '
.*:54: Error: unrecognized extended version of MIPS16 opcode `move\.e \$0,\$16'
.*:55: Error: unrecognized extended version of MIPS16 opcode `move\.e \$16,\$0'
.*:57: Warning: extended operand requested but not required
.*:59: Warning: extended operand requested but not required
.*:60: Warning: extended operand requested but not required
.*:62: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `sd\.e \$16,0\(\$16\)'
.*:64: Warning: extended operand requested but not required
.*:66: Warning: extended operand requested but not required
.*:68: Warning: extended operand requested but not required
.*:70: Warning: extended operand requested but not required
.*:72: Warning: extended operand requested but not required
.*:74: Warning: extended operand requested but not required
.*:77: Warning: extended operand requested but not required
.*:80: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `lwu\.e \$16,0\(\$16\)'
.*:82: Warning: extended operand requested but not required
.*:84: Warning: extended operand requested but not required
.*:86: Warning: extended operand requested but not required
.*:88: Warning: extended operand requested but not required
.*:90: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddu\.e \$16,\$16,\$16'
.*:91: Error: operand 3 must be an immediate expression `addu\.e \$16,\$16,\$16'
.*:92: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsubu\.e \$16,\$16,\$16'
.*:93: Error: unrecognized extended version of MIPS16 opcode `subu\.e \$16,\$16,\$16'
.*:95: Error: unrecognized extended version of MIPS16 opcode `jr\.e \$16'
.*:97: Error: unrecognized extended version of MIPS16 opcode `j\.e \$16'
.*:99: Error: unrecognized extended version of MIPS16 opcode `jr\.e \$31'
.*:101: Error: unrecognized extended version of MIPS16 opcode `j\.e \$31'
.*:103: Error: unrecognized extended version of MIPS16 opcode `jalr\.e \$16'
.*:105: Error: unrecognized extended version of MIPS16 opcode `jalr\.e \$31,\$16'
.*:107: Error: operand 1 must be an immediate expression `jal\.e \$16'
.*:109: Error: operand 1 must be an immediate expression `jal\.e \$31,\$16'
.*:111: Error: unrecognized extended version of MIPS16 opcode `jrc\.e \$16'
.*:112: Error: unrecognized extended version of MIPS16 opcode `jrc\.e \$31'
.*:113: Error: unrecognized extended version of MIPS16 opcode `jalrc\.e \$16'
.*:114: Error: unrecognized extended version of MIPS16 opcode `jalrc\.e \$31,\$16'
.*:115: Error: unrecognized extended version of MIPS16 opcode `sdbbp\.e 0'
.*:116: Error: operand 2 must be an immediate expression `slt\.e \$16,\$16'
.*:117: Error: operand 2 must be an immediate expression `sltu\.e \$16,\$16'
.*:118: Error: unrecognized extended version of MIPS16 opcode `sllv\.e \$16,\$16'
.*:119: Error: operand 2 must be an immediate expression `sll\.e \$16,\$16'
.*:120: Error: unrecognized extended version of MIPS16 opcode `break\.e 0'
.*:121: Error: unrecognized extended version of MIPS16 opcode `srlv\.e \$16,\$16'
.*:122: Error: operand 2 must be an immediate expression `srl\.e \$16,\$16'
.*:123: Error: unrecognized extended version of MIPS16 opcode `srav\.e \$16,\$16'
.*:124: Error: operand 2 must be an immediate expression `sra\.e \$16,\$16'
.*:125: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsrl\.e \$16,8'
.*:126: Error: unrecognized extended version of MIPS16 opcode `entry\.e '
.*:127: Error: unrecognized extended version of MIPS16 opcode `entry\.e \$31'
.*:128: Error: unrecognized extended version of MIPS16 opcode `exit\.e \$f0'
.*:129: Error: unrecognized extended version of MIPS16 opcode `exit\.e'
.*:130: Error: operand 2 must be an immediate expression `cmp\.e \$16,\$16'
.*:131: Error: unrecognized extended version of MIPS16 opcode `neg\.e \$16,\$16'
.*:132: Error: unrecognized extended version of MIPS16 opcode `and\.e \$16,\$16'
.*:133: Error: unrecognized extended version of MIPS16 opcode `or\.e \$16,\$16'
.*:134: Error: unrecognized extended version of MIPS16 opcode `xor\.e \$16,\$16'
.*:135: Error: unrecognized extended version of MIPS16 opcode `not\.e \$16,\$16'
.*:136: Error: unrecognized extended version of MIPS16 opcode `mfhi\.e \$16'
.*:137: Error: unrecognized extended version of MIPS16 opcode `zeb\.e \$16'
.*:138: Error: unrecognized extended version of MIPS16 opcode `zeh\.e \$16'
.*:139: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `zew\.e \$16'
.*:140: Error: unrecognized extended version of MIPS16 opcode `seb\.e \$16'
.*:141: Error: unrecognized extended version of MIPS16 opcode `seh\.e \$16'
.*:142: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `sew\.e \$16'
.*:143: Error: unrecognized extended version of MIPS16 opcode `mflo\.e \$16'
.*:144: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsra\.e \$16,8'
.*:145: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsllv\.e \$16,\$16'
.*:146: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsll\.e \$16,\$16'
.*:147: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsrlv\.e \$16,\$16'
.*:148: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsrl\.e \$16,\$16'
.*:149: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsrav\.e \$16,\$16'
.*:150: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dsra\.e \$16,\$16'
.*:151: Error: unrecognized extended version of MIPS16 opcode `mult\.e \$16,\$16'
.*:152: Error: unrecognized extended version of MIPS16 opcode `multu\.e \$16,\$16'
.*:153: Error: unrecognized extended version of MIPS16 opcode `div\.e \$0,\$16,\$16'
.*:154: Error: unrecognized extended version of MIPS16 opcode `rem\.e \$0,\$16,\$16'
.*:155: Error: unrecognized extended version of MIPS16 opcode `divu\.e \$0,\$16,\$16'
.*:156: Error: unrecognized extended version of MIPS16 opcode `remu\.e \$0,\$16,\$16'
.*:157: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dmult\.e \$16,\$16'
.*:158: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dmultu\.e \$16,\$16'
.*:159: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `ddiv\.e \$0,\$16,\$16'
.*:160: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `drem\.e \$0,\$16,\$16'
.*:161: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `ddivu\.e \$0,\$16,\$16'
.*:162: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dremu\.e \$0,\$16,\$16'
.*:164: Error: unrecognized extended version of MIPS16 opcode `extend\.e 0'
.*:166: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `ld\.e \$16,0\(\$29\)'
.*:167: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `sd\.e \$16,0\(\$29\)'
.*:168: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `sd\.e \$31,0\(\$29\)'
.*:169: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddiu\.e \$29,0'
.*:170: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddiu\.e \$29,\$29,0'
.*:171: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddu\.e \$29,0'
.*:172: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddu\.e \$29,\$29,0'
.*:174: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `ld\.e \$16,0\(\$pc\)'
.*:175: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `ld\.e \$16,\.-3'
.*:176: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddiu\.e \$16,0'
.*:177: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddu\.e \$16,0'
.*:179: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddiu\.e \$16,\$pc,0'
.*:180: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddu\.e \$16,\$pc,0'
.*:181: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `dla\.e \$16,\.-1'
.*:182: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddiu\.e \$16,\$sp,0'
.*:183: Error: opcode not supported on this processor: interaptiv-mr2 \(mips32r3\) `daddu\.e \$16,\$sp,0'
.*:10: Warning: extended operand requested but not required
.*:12: Warning: extended operand requested but not required
.*:19: Warning: extended operand requested but not required
.*:21: Warning: extended operand requested but not required
.*:44: Warning: extended operand requested but not required
.*:45: Warning: extended operand requested but not required
