.*: Assembler messages:
.*:5: Error: operand value out of range for instruction
.*:6: Error: operand value out of range for instruction
.*:7: Error: operand 2 out of range `addiu \$2,0x10000000000000000'
.*:8: Error: operand 2 must be an immediate expression `addiu \$2,\$3'
.*:9: Error: invalid operands `addiu \$2,\(\$3\)'
.*:10: Error: register value used as expression `addiu \$2,0\+\$3'
.*:11: Error: register value used as expression `addiu \$2,\(\(\$3\)\)'
