# name: attributes for -mcpu=hs34
# source: blank.s
# as: -mcpu=hs34
# readelf: -A

Attribute Section: ARC
File Attributes
  Tag_ARC_CPU_base: ARCHS
  Tag_ARC_CPU_name: "hs34"
  Tag_ARC_ABI_osver: v4
  Tag_ARC_ISA_config: "CD"
  Tag_ARC_ATR_version: 1
