target:
	th.flrd		fa0, a1, a2, 0
	th.flrd		fa0, a1, a2, 3
	th.flrw		fa0, a1, a2, 0
	th.flrw		fa0, a1, a2, 3
	th.flurd	fa0, a1, a2, 0
	th.flurd	fa0, a1, a2, 3
	th.flurw	fa0, a1, a2, 0
	th.flurw	fa0, a1, a2, 3
	th.fsrd		fa0, a1, a2, 0
	th.fsrd		fa0, a1, a2, 3
	th.fsrw		fa0, a1, a2, 0
	th.fsrw		fa0, a1, a2, 3
	th.fsurd	fa0, a1, a2, 0
	th.fsurd	fa0, a1, a2, 3
	th.fsurw	fa0, a1, a2, 0
	th.fsurw	fa0, a1, a2, 3
