.*: Assembler messages:
.*:8: Error: unsupported relocation
.*:9: Error: unsupported relocation
.*:10: Error: unsupported relocation
.*:11: Error: unsupported relocation
.*:12: Error: unsupported relocation
.*:13: Error: unsupported relocation
.*:14: Error: unsupported relocation
.*:15: Error: unsupported relocation
