        .text
        ;; schd
        schd.rw
        schd.rd
        schd.wft
        schd.wft.ie1
        schd.wft.ie2
        schd.wft.ie12

        ;; sync
        sync.rd
        sync.wr

        ;; hwschd
        hwschd.off      r10
        hwschd.restore  0, r20
