--  Copyright 2012-2023 Free Software Foundation, Inc.
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.

procedure Foo is
   type Discriminants_Record (A : Integer; B : Boolean) is record
      C : Float;
   end record;
   --  The following variable is unused on purpose, and might be
   --  optimized out by the compiler.
   Z : Discriminants_Record := (A => 1, B => False, C => 2.0);
begin
   null;
end Foo;
