.*: Assembler messages:
.*:4: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$0'
.*:5: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$1'
.*:6: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$2'
.*:7: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$3'
.*:8: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$4'
.*:9: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$5'
.*:10: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$6'
.*:11: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$7'
.*:12: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$8'
.*:13: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$9'
.*:14: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$10'
.*:15: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$11'
.*:16: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$12'
.*:17: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$13'
.*:18: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$14'
.*:19: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$15'
.*:20: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$16'
.*:21: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$17'
.*:22: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$18'
.*:23: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$19'
.*:24: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$20'
.*:25: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$21'
.*:26: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$22'
.*:27: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$23'
.*:28: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$24'
.*:29: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$25'
.*:30: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$26'
.*:31: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$27'
.*:32: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$28'
.*:33: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$29'
.*:34: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$30'
.*:35: Error: opcode not supported on this processor: .* \(.*\) `dmtc2 \$0,\$31'
.*:37: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$0'
.*:38: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$1'
.*:39: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$2'
.*:40: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$3'
.*:41: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$4'
.*:42: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$5'
.*:43: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$6'
.*:44: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$7'
.*:45: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$8'
.*:46: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$9'
.*:47: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$10'
.*:48: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$11'
.*:49: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$12'
.*:50: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$13'
.*:51: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$14'
.*:52: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$15'
.*:53: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$16'
.*:54: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$17'
.*:55: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$18'
.*:56: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$19'
.*:57: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$20'
.*:58: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$21'
.*:59: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$22'
.*:60: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$23'
.*:61: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$24'
.*:62: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$25'
.*:63: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$26'
.*:64: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$27'
.*:65: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$28'
.*:66: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$29'
.*:67: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$30'
.*:68: Error: opcode not supported on this processor: .* \(.*\) `dmfc2 \$0,\$31'
