.*: Assembler messages:
.*:6: Warning: relaxed out-of-range branch into a jump
.*:11: Warning: relaxed out-of-range branch into a jump
.*:7: Error: branch out of range
.*:8: Error: branch out of range
.*:9: Error: branch out of range
.*:10: Error: branch out of range
