.*: Assembler messages:
.*:5: Error: operand 4 out of range `ext \$2,\$3,1,0'
.*:6: Error: operand 4 out of range `dext \$2,\$3,1,0'
.*:7: Error: operand 4 out of range `dextm \$2,\$3,31,2'
.*:8: Error: operand 4 out of range `dextm \$2,\$3,1,32'
.*:9: Error: operand 4 out of range `dextu \$2,\$3,33,0'
