#objdump: -d
#name:    
#source:  sat.s


.*:     file format elf32-s12z


Disassembly of section .text:

00000000 <.text>:
   0:	1b a0       	sat d2
   2:	1b a1       	sat d3
   4:	1b a2       	sat d4
   6:	1b a3       	sat d5
   8:	1b a4       	sat d0
   a:	1b a5       	sat d1
   c:	1b a6       	sat d6
   e:	1b a7       	sat d7
